from setuptools import setup, find_packages

setup(
    name='strictconf',
    version='0.3.0rc2',
    description='Configuration library with type-checking',
    author='Vladimir Magamedov',
    author_email='vladimir@magamedov.com',
    url='https://github.com/vmagamedov/strictconf',
    packages=find_packages(),
    license='BSD',
)
